@@ -55,7 +55,8 @@ class GrlTrackerPlaylists(GObject.GObject):
     ]
 
     def __init__(
-            self, source, coremodel, application, grilo, tracker_wrapper):
+            self, source, coremodel, application, grilo, tracker_wrapper,
+            songs_hash):
         """Initialize GrlTrackerPlaylists.
 
         :param Grl.TrackerSource source: The Tracker source to wrap
@@ -78,6 +79,7 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._model_filter = self._coremodel.props.playlists_filter
         self._user_model_filter = self._coremodel.props.user_playlists_filter
         self._pls_todelete = []
+        self._songs_hash = songs_hash
         self._tracker = tracker_wrapper.props.tracker
         self._tracker_wrapper = tracker_wrapper
         self._window = application.props.window
@@ -95,7 +97,8 @@ class GrlTrackerPlaylists(GObject.GObject):
             "source": self._source,
             "application": self._application,
             "grilo": self._grilo,
-            "tracker_wrapper": self._tracker_wrapper
+            "tracker_wrapper": self._tracker_wrapper,
+            "songs_hash": self._songs_hash
         }
 
         smart_playlists = {
@@ -149,7 +152,7 @@ class GrlTrackerPlaylists(GObject.GObject):
         playlist = Playlist(
             media=media, source=self._source, coremodel=self._coremodel,
             application=self._application, grilo=self._grilo,
-            tracker_wrapper=self._tracker_wrapper)
+            tracker_wrapper=self._tracker_wrapper, songs_hash=self._songs_hash)
 
         self._model.append(playlist)
         callback = data
@@ -257,6 +260,20 @@ class GrlTrackerPlaylists(GObject.GObject):
         self._tracker.update_blank_async(
             query, GLib.PRIORITY_LOW, None, _create_cb, None)
 
+    def check_smart_playlist_change(self):
+        """Check if smart playlists need to be updated.
+
+        A smart playlist needs to be updated in two cases:
+        * it is being played (active_playlist)
+        * it is visible in PlaylistsView
+        """
+        active_playlist = self._coremodel.props.active_playlist
+        if (active_playlist is not None
+                and active_playlist.props.is_smart is True):
+            active_playlist.update()
+        else:
+            self._coremodel.emit("smart-playlist-change")
+
 
 class Playlist(GObject.GObject):
     """ Base class of all playlists """
@@ -292,7 +309,7 @@ class Playlist(GObject.GObject):
     def __init__(
             self, media=None, query=None, tag_text=None, source=None,
             coremodel=None, application=None, grilo=None,
-            tracker_wrapper=None):
+            tracker_wrapper=None, songs_hash=None):
 
         super().__init__()
         """Initialize a playlist
@@ -324,6 +341,7 @@ class Playlist(GObject.GObject):
         self._coreselection = application.props.coreselection
         self._grilo = grilo
         self._log = application.props.log
+        self._songs_hash = songs_hash
         self._tracker = tracker_wrapper.props.tracker
         self._tracker_wrapper = tracker_wrapper
         self._window = application.props.window
@@ -354,7 +372,7 @@ class Playlist(GObject.GObject):
         SELECT
             rdf:type(?song)
             ?song AS ?tracker_urn
-            tracker:id(?entry) AS ?id
+            tracker:id(?song) AS ?id
             nie:url(?song) AS ?url
             nie:title(?song) AS ?title
             nmm:artistName(nmm:performer(?song)) AS ?artist
@@ -398,6 +416,7 @@ class Playlist(GObject.GObject):
                 return
 
             coresong = CoreSong(media, self._coreselection, self._grilo)
+            self._bind_to_main_song(coresong)
             if coresong not in self._songs_todelete:
                 self._model.append(coresong)
 
@@ -408,6 +427,20 @@ class Playlist(GObject.GObject):
         self._source.query(
             query, self.METADATA_KEYS, options, _add_to_playlist_cb, None)
 
+    def _bind_to_main_song(self, coresong):
+        main_coresong = self._songs_hash[coresong.props.media.get_id()]
+
+        properties = [
+            "album", "album_disc_number", "artist", "duration", "media",
+            "grlid", "play_count", "state", "title", "track_number", "url",
+            "validation", "favorite", "selected"]
+
+        for prop in properties:
+            main_coresong.bind_property(
+                prop, coresong, prop,
+                GObject.BindingFlags.BIDIRECTIONAL
+                | GObject.BindingFlags.SYNC_CREATE)
+
     @GObject.Property(type=str, default=None)
     def title(self):
         """Playlist title
@@ -570,6 +603,7 @@ class Playlist(GObject.GObject):
                 return
 
             coresong = CoreSong(media, self._coreselection, self._grilo)
+            self._bind_to_main_song(coresong)
             if coresong not in self._songs_todelete:
                 self._model.append(coresong)
 
@@ -582,7 +616,7 @@ class Playlist(GObject.GObject):
             SELECT
                 rdf:type(?song)
                 ?song AS ?tracker_urn
-                tracker:id(?entry) AS ?id
+                tracker:id(?song) AS ?id
                 nie:url(?song) AS ?url
                 nie:title(?song) AS ?title
                 nmm:artistName(nmm:performer(?song)) AS ?artist
@@ -720,6 +754,7 @@ class SmartPlaylist(Playlist):
                     return
 
                 coresong = CoreSong(media, self._coreselection, self._grilo)
+                self._bind_to_main_song(coresong)
                 self._model.append(coresong)
 
             options = self._fast_options.copy()
@@ -729,6 +764,52 @@ class SmartPlaylist(Playlist):
 
         return self._model
 
+    def update(self):
+        """Updates playlist model."""
+        if self._model is None:
+            return
+
+        new_model_medias = []
+
+        def _fill_new_model(source, op_id, media, remaining, error):
+            if error:
+                return
+
+            if not media:
+                self._finish_update(new_model_medias)
+                return
+
+            new_model_medias.append(media)
+
+        options = self._fast_options.copy()
+        self._source.query(
+            self.props.query, self.METADATA_KEYS, options, _fill_new_model)
+
+    def _finish_update(self, new_model_medias):
+        if not new_model_medias:
+            self._model.remove_all()
+            return
+
+        current_models_ids = [coresong.props.media.get_id()
+                              for coresong in self._model]
+        new_model_ids = [media.get_id() for media in new_model_medias]
+
+        idx_to_delete = []
+        for idx, media_id in enumerate(current_models_ids):
+            if media_id not in new_model_ids:
+                idx_to_delete.insert(0, idx)
+
+        for idx in idx_to_delete:
+            self._model.remove(idx)
+            self.props.count -= 1
+
+        for idx, media in enumerate(new_model_medias):
+            if media.get_id() not in current_models_ids:
+                coresong = CoreSong(media, self._coreselection, self._grilo)
+                self._bind_to_main_song(coresong)
+                self._model.append(coresong)
+                self.props.count += 1
+
 
 class MostPlayed(SmartPlaylist):
     """Most Played smart playlist"""
