@@ -74,10 +74,11 @@ class GrlTrackerWrapper(GObject.GObject):
         """
         super().__init__()
 
+        self._application = application
         self._coremodel = coremodel
         self._coreselection = application.props.coreselection
         self._grilo = grilo
-        self._source = source
+        self._source = None
         self._log = application.props.log
         self._model = self._coremodel.props.songs
         self._albums_model = self._coremodel.props.albums
@@ -90,6 +91,7 @@ class GrlTrackerWrapper(GObject.GObject):
         self._artist_search_model = self._coremodel.props.artists_search
         self._batch_changed_media_ids = {}
         self._content_changed_timeout = None
+        self._tracker_playlists = None
         self._tracker_wrapper = tracker_wrapper
         self._window = application.props.window
 
@@ -101,21 +103,35 @@ class GrlTrackerWrapper(GObject.GObject):
         self._fast_options.set_resolution_flags(
             Grl.ResolutionFlags.FAST_ONLY | Grl.ResolutionFlags.IDLE_RELAY)
 
-        self._initial_songs_fill(self._source)
-        self._initial_albums_fill(self._source)
-        self._initial_artists_fill(self._source)
+        self._content_changed_id = None
+        self.props.source = source
 
-        self._tracker_playlists = GrlTrackerPlaylists(
-            source, coremodel, application, grilo, tracker_wrapper)
+        self._initial_songs_fill(self.props.source)
+        self._initial_albums_fill(self.props.source)
+        self._initial_artists_fill(self.props.source)
 
-        self._source.notify_change_start()
-        self._source.connect("content-changed", self._batch_content_changed)
-
-    @GObject.Property(
-        type=Grl.Source, default=None, flags=GObject.ParamFlags.READABLE)
+    @GObject.Property(type=Grl.Source, default=None)
     def source(self):
         return self._source
 
+    @source.setter
+    def source(self, new_source):
+        """Set a new grilo tracker source
+
+        Everytime, the tracker plugin is loaded, a new source is
+        created. The source needs to be updated to get notifications.
+
+        :param Grl.Source new_source: new grilo tracker source
+        """
+        if self._content_changed_id is not None:
+            self._source.disconnect(self._content_changed_id)
+            self._content_changed_id = None
+
+        self._source = new_source
+        self._source.notify_change_start()
+        self._content_changed_id = self._source.connect(
+            "content-changed", self._batch_content_changed)
+
     def _batch_content_changed(self, source, medias, change_type, loc_unknown):
         if medias == []:
             return
@@ -149,6 +165,7 @@ class GrlTrackerWrapper(GObject.GObject):
 
         self._check_album_change()
         self._check_artist_change()
+        self._tracker_playlists.check_smart_playlist_change()
 
         self._batch_changed_media_ids = {}
         self._content_changed_timeout = None
@@ -212,7 +229,7 @@ class GrlTrackerWrapper(GObject.GObject):
 
         options = self._fast_options.copy()
 
-        self._source.query(
+        self.props.source.query(
             query, self.METADATA_KEYS, options, check_album_cb)
 
     def _check_artist_change(self):
@@ -272,7 +289,7 @@ class GrlTrackerWrapper(GObject.GObject):
 
         options = self._fast_options.copy()
 
-        self._source.query(
+        self.props.source.query(
             query, self.METADATA_KEYS, options, check_artist_cb)
 
     def _remove_media(self, media_ids):
@@ -348,7 +365,7 @@ class GrlTrackerWrapper(GObject.GObject):
 
         options = self._fast_options.copy()
 
-        self._source.query(
+        self.props.source.query(
             self._song_media_query(media_ids), self.METADATA_KEYS,
             options, _update_changed_media)
 
@@ -365,6 +382,14 @@ class GrlTrackerWrapper(GObject.GObject):
             if not media:
                 self._model.splice(self._model.get_n_items(), 0, songs_added)
                 self._window.notifications_popup.pop_loading()
+
+                # Initialize the playlists subwrapper after the initial
+                # songs model fill, the playlists expect a filled songs
+                # hashtable.
+                self._tracker_playlists = GrlTrackerPlaylists(
+                    self.props.source, self._coremodel, self._application,
+                    self._grilo, self._tracker_wrapper, self._hash)
+
                 return
 
             song = CoreSong(media, self._coreselection, self._grilo)
@@ -404,7 +429,8 @@ class GrlTrackerWrapper(GObject.GObject):
         }
 
         options = self._fast_options.copy()
-        self._source.query(query, self.METADATA_KEYS, options, _add_to_model)
+        self.props.source.query(
+            query, self.METADATA_KEYS, options, _add_to_model)
 
     def _initial_albums_fill(self, source):
         self._window.notifications_popup.push_loading()
@@ -568,7 +594,7 @@ class GrlTrackerWrapper(GObject.GObject):
             return False
 
         options = self._fast_options.copy()
-        self._source.query(
+        self.props.source.query(
             query, [Grl.METADATA_KEY_TITLE], options, query_cb)
 
     def get_album_discs(self, media, disc_model):
@@ -612,7 +638,7 @@ class GrlTrackerWrapper(GObject.GObject):
             disc_model.append(coredisc)
 
         options = self._fast_options.copy()
-        self._source.query(
+        self.props.source.query(
             query, [Grl.METADATA_KEY_ALBUM_DISC_NUMBER], options, _disc_nr_cb)
 
     def populate_album_disc_songs(self, media, disc_nr, callback):
@@ -659,7 +685,7 @@ class GrlTrackerWrapper(GObject.GObject):
         }
 
         options = self._fast_options.copy()
-        self._source.query(query, self.METADATA_KEYS, options, callback)
+        self.props.source.query(query, self.METADATA_KEYS, options, callback)
 
     def search(self, text):
         # FIXME: Searches are limited to not bog down the UI with
@@ -725,7 +751,7 @@ class GrlTrackerWrapper(GObject.GObject):
             artist_filter_ids.append(media.get_id())
 
         options = self._fast_options.copy()
-        self._source.query(
+        self.props.source.query(
             query, self.METADATA_KEYS, options, artist_search_cb)
 
         # Album search
@@ -780,7 +806,7 @@ class GrlTrackerWrapper(GObject.GObject):
             album_filter_ids.append(media.get_id())
 
         options = self._fast_options.copy()
-        self._source.query(
+        self.props.source.query(
             query, self.METADATA_KEYS, options, albums_search_cb)
 
         # Song search
@@ -842,7 +868,8 @@ class GrlTrackerWrapper(GObject.GObject):
 
         options = self._fast_options.copy()
 
-        self._source.query(query, self.METADATA_KEYS, options, songs_search_cb)
+        self.props.source.query(
+            query, self.METADATA_KEYS, options, songs_search_cb)
 
     def get_album_art_for_item(self, coresong, callback):
         """Placeholder until we got a better solution
@@ -860,7 +887,7 @@ class GrlTrackerWrapper(GObject.GObject):
             | Grl.ResolutionFlags.IDLE_RELAY)
         full_options.set_count(1)
 
-        self._source.query(
+        self.props.source.query(
             query, self.METADATA_THUMBNAIL_KEYS, full_options, callback)
 
     def _get_album_for_album_id(self, album_id):
@@ -963,7 +990,7 @@ class GrlTrackerWrapper(GObject.GObject):
         full_options.set_resolution_flags(
             Grl.ResolutionFlags.FULL | Grl.ResolutionFlags.IDLE_RELAY)
 
-        self._source.resolve(
+        self.props.source.resolve(
             media, [Grl.METADATA_KEY_THUMBNAIL], full_options, _resolve_cb,
             None)
 
