@@ -22,17 +22,13 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import logging
 from gettext import gettext as _
 from gi.repository import Gdk, Gtk
 
-from gnomemusic import log
 from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.artistalbumswidget import ArtistAlbumsWidget
 from gnomemusic.widgets.artisttile import ArtistTile
 
-logger = logging.getLogger(__name__)
-
 
 class ArtistsView(BaseView):
     """Main view of all available artists
@@ -41,15 +37,10 @@ class ArtistsView(BaseView):
     all albums by this artist on the right side.
     """
 
-    def __repr__(self):
-        return '<ArtistsView>'
-
-    @log
-    def __init__(self, application, player):
+    def __init__(self, application):
         """Initialize
 
         :param GtkApplication application: The application object
-        :param player: The main player object
         """
         self._sidebar = Gtk.ListBox()
         sidebar_container = Gtk.ScrolledWindow()
@@ -61,6 +52,8 @@ class ArtistsView(BaseView):
         self._application = application
         self._artists = {}
 
+        self._selected_artist = None
+
         self._window = application.props.window
         self._coremodel = application.props.coremodel
         self._model = self._coremodel.props.artists_sort
@@ -114,7 +107,7 @@ class ArtistsView(BaseView):
                         or self._sidebar.get_row_at_index(position - 1))
             if row_next:
                 self._sidebar.select_row(row_next)
-                row_next.emit("activate")
+                self._on_artist_activated(self._sidebar, row_next)
 
         removed_frame = self._view.get_child_by_name(removed_artist)
         self._view.remove(removed_frame)
@@ -126,9 +119,8 @@ class ArtistsView(BaseView):
             return
 
         self._sidebar.select_row(first_row)
-        first_row.emit("activate")
+        self._on_artist_activated(self._sidebar, first_row)
 
-    @log
     def _setup_view(self):
         self._view_container = Gtk.ScrolledWindow(hexpand=True, vexpand=True)
         self._box.pack_start(self._view_container, True, True, 0)
@@ -138,10 +130,8 @@ class ArtistsView(BaseView):
             vhomogeneous=False)
         self._view_container.add(self._view)
 
-    @log
     def _on_artist_activated(self, sidebar, row, data=None):
         """Initializes new artist album widgets"""
-        artist_tile = row.get_child()
         # On application start the first row of ArtistView is activated
         # to show an intial artist. When this happens while any of the
         # views are in selection mode, this artist will be incorrectly
@@ -150,11 +140,14 @@ class ArtistsView(BaseView):
         # ArtistsView, to circumvent this issue.
         if (self.props.selection_mode
                 and self._window.props.active_view is self):
-            artist_tile.props.selected = not artist_tile.props.selected
+            row.props.selected = not row.props.selected
             return
 
+        selected_row = self._sidebar.get_selected_row()
+        self._selected_artist = selected_row.props.coreartist
+
         # Prepare a new artist_albums_widget here
-        coreartist = artist_tile.props.coreartist
+        coreartist = row.props.coreartist
         if coreartist.props.artist in self._loaded_artists:
             scroll_vadjustment = self._view_container.props.vadjustment
             scroll_vadjustment.props.value = 0.
@@ -175,7 +168,6 @@ class ArtistsView(BaseView):
 
         self._loaded_artists.append(coreartist.props.artist)
 
-    @log
     def _on_sidebar_clicked(self, gesture, n_press, x, y):
         success, state = Gtk.get_current_event_state()
         modifiers = Gtk.accelerator_get_default_mod_mask()
@@ -183,12 +175,10 @@ class ArtistsView(BaseView):
                 and not self.props.selection_mode):
             self.props.selection_mode = True
 
-    @log
-    def _on_selection_changed(self, widget, value, data=None):
-        return
-
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
+        if self.get_parent().get_visible_child() != self:
+            return
+
         super()._on_selection_mode_changed(widget, data)
 
         self._view.props.sensitive = not self.props.selection_mode
@@ -196,20 +186,28 @@ class ArtistsView(BaseView):
             self._sidebar.props.selection_mode = Gtk.SelectionMode.NONE
         else:
             self._sidebar.props.selection_mode = Gtk.SelectionMode.SINGLE
+            selected_row = self._sidebar.get_row_at_index(0)
+            if selected_row is None:
+                self._selected_artist = None
+                return
+
+            for row in self._sidebar:
+                if row.props.coreartist == self._selected_artist:
+                    selected_row = row
+                    break
+
+            self._sidebar.select_row(selected_row)
+            self._selected_artist = None
 
-    @log
     def _toggle_all_selection(self, selected):
 
         def toggle_selection(child):
-            tile = child.get_child()
-            tile.props.selected = selected
+            child.props.selected = selected
 
         self._sidebar.foreach(toggle_selection)
 
-    @log
     def select_all(self):
         self._toggle_all_selection(True)
 
-    @log
-    def unselect_all(self):
+    def deselect_all(self):
         self._toggle_all_selection(False)
