@@ -22,19 +22,15 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-import logging
 from gettext import gettext as _
 from gi.repository import Gdk, Gtk, Pango
 
-from gnomemusic import log
 from gnomemusic.coresong import CoreSong
 from gnomemusic.player import PlayerPlaylist
 from gnomemusic.utils import SongStateIcon
 from gnomemusic.views.baseview import BaseView
 from gnomemusic.widgets.starhandlerwidget import StarHandlerWidget
 
-logger = logging.getLogger(__name__)
-
 
 class SongsView(BaseView):
     """Main view of all songs sorted artistwise
@@ -43,15 +39,10 @@ class SongsView(BaseView):
     and the album name.
     """
 
-    def __repr__(self):
-        return '<SongsView>'
-
-    @log
-    def __init__(self, application, player):
+    def __init__(self, application):
         """Initialize
 
         :param GtkApplication window: The application object
-        :param player: The main player object
         """
         self._coremodel = application.props.coremodel
         super().__init__('songs', _("Songs"), application)
@@ -64,13 +55,12 @@ class SongsView(BaseView):
 
         self._playlist_model = self._coremodel.props.playlist_sort
 
-        self._player = player
+        self._player = application.props.player
         self._player.connect('song-changed', self._update_model)
 
         self._model = self._view.props.model
         self._view.show()
 
-    @log
     def _setup_view(self):
         view_container = Gtk.ScrolledWindow(hexpand=True, vexpand=True)
         self._box.pack_start(view_container, True, True, 0)
@@ -90,7 +80,6 @@ class SongsView(BaseView):
 
         view_container.add(self._view)
 
-    @log
     def _add_list_renderers(self):
         now_playing_symbol_renderer = Gtk.CellRendererPixbuf(
             xpad=0, xalign=0.5, yalign=0.5)
@@ -128,7 +117,9 @@ class SongsView(BaseView):
         column_album.props.expand = True
         self._view.append_column(column_album)
 
-        duration_renderer = Gtk.CellRendererText(xalign=1.0)
+        attrs = Pango.AttrList()
+        attrs.insert(Pango.AttrFontFeatures.new("tnum=1"))
+        duration_renderer = Gtk.CellRendererText(xalign=1.0, attributes=attrs)
         column_duration = Gtk.TreeViewColumn(
             "Duration", duration_renderer, text=5)
         self._view.append_column(column_duration)
@@ -153,14 +144,15 @@ class SongsView(BaseView):
         else:
             cell.props.visible = False
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
+        if self.get_parent().get_visible_child() != self:
+            return
+
         super()._on_selection_mode_changed(widget, data)
 
         cols = self._view.get_columns()
         cols[1].props.visible = self.props.selection_mode
 
-    @log
     def _on_item_activated(self, treeview, path, column):
         """Action performed when clicking on a song
 
@@ -185,7 +177,6 @@ class SongsView(BaseView):
 
         self._player.play(coresong)
 
-    @log
     def _on_view_clicked(self, gesture, n_press, x, y):
         """Ctrl+click on self._view triggers selection mode."""
         _, state = Gtk.get_current_event_state()
@@ -206,7 +197,6 @@ class SongsView(BaseView):
             self._model[iter_][1] = new_fav_status
             self._model[iter_][7].props.selected = new_fav_status
 
-    @log
     def _update_model(self, player):
         """Updates model when the song changes
 
@@ -245,5 +235,5 @@ class SongsView(BaseView):
     def select_all(self):
         self._select(True)
 
-    def unselect_all(self):
+    def deselect_all(self):
         self._select(False)
