@@ -25,7 +25,6 @@
 from gi.repository import Gtk, Gdk, Gio, GLib, GObject
 from gettext import gettext as _
 
-from gnomemusic import log
 from gnomemusic.gstplayer import Playback
 from gnomemusic.mediakeys import MediaKeys
 from gnomemusic.player import RepeatMode
@@ -46,9 +45,6 @@ from gnomemusic.widgets.searchheaderbar import SearchHeaderBar
 from gnomemusic.widgets.selectiontoolbar import SelectionToolbar  # noqa: F401
 from gnomemusic.windowplacement import WindowPlacement
 
-import logging
-logger = logging.getLogger(__name__)
-
 
 @Gtk.Template(resource_path="/org/gnome/Music/ui/Window.ui")
 class Window(Gtk.ApplicationWindow):
@@ -65,10 +61,6 @@ class Window(Gtk.ApplicationWindow):
     _selection_toolbar = Gtk.Template.Child()
     _stack = Gtk.Template.Child()
 
-    def __repr__(self):
-        return '<Window>'
-
-    @log
     def __init__(self, app):
         """Initialize the main window.
 
@@ -87,9 +79,9 @@ class Window(Gtk.ApplicationWindow):
         select_all = Gio.SimpleAction.new('select_all', None)
         select_all.connect('activate', self._select_all)
         self.add_action(select_all)
-        select_none = Gio.SimpleAction.new('select_none', None)
-        select_none.connect('activate', self._select_none)
-        self.add_action(select_none)
+        deselect_all = Gio.SimpleAction.new('deselect_all', None)
+        deselect_all.connect('activate', self._deselect_all)
+        self.add_action(deselect_all)
 
         self.set_size_request(200, 100)
         WindowPlacement(self)
@@ -103,13 +95,12 @@ class Window(Gtk.ApplicationWindow):
 
         MediaKeys(self._player, self)
 
-    @log
     def _setup_view(self):
         self._search = Search()
         self._headerbar_stack = Gtk.Stack()
         transition_type = Gtk.StackTransitionType.CROSSFADE
         self._headerbar_stack.props.transition_type = transition_type
-        self._headerbar = HeaderBar()
+        self._headerbar = HeaderBar(self._app)
         self._search_headerbar = SearchHeaderBar(self._app)
         self._search_headerbar.props.stack = self._stack
         self._headerbar_stack.add_named(self._headerbar, "main")
@@ -196,7 +187,6 @@ class Window(Gtk.ApplicationWindow):
         else:
             self._switch_to_empty_view()
 
-    @log
     def _switch_to_empty_view(self):
         did_initial_state = self._settings.get_boolean('did-initial-state')
 
@@ -238,7 +228,6 @@ class Window(Gtk.ApplicationWindow):
 
         self._on_songs_available(None, None)
 
-    @log
     def _switch_to_player_view(self):
         self._settings.set_boolean('did-initial-state', True)
         self._on_notify_model_id = self._stack.connect(
@@ -267,16 +256,11 @@ class Window(Gtk.ApplicationWindow):
         self._headerbar.props.state = HeaderBar.State.MAIN
         self._headerbar.props.stack = self._stack
 
-        self.views[View.ALBUM] = AlbumsView(self._app, self._player)
-        self.views[View.ARTIST] = ArtistsView(self._app, self._player)
-        self.views[View.SONG] = SongsView(self._app, self._player)
-        self.views[View.PLAYLIST] = PlaylistsView(self._app, self._player)
-        self.views[View.SEARCH] = SearchView(self._app, self._player)
-
-        selectable_views = [View.ALBUM, View.ARTIST, View.SONG, View.SEARCH]
-        for view in selectable_views:
-            self.views[view].bind_property(
-                'selected-items-count', self, 'selected-items-count')
+        self.views[View.ALBUM] = AlbumsView(self._app)
+        self.views[View.ARTIST] = ArtistsView(self._app)
+        self.views[View.SONG] = SongsView(self._app)
+        self.views[View.PLAYLIST] = PlaylistsView(self._app)
+        self.views[View.SEARCH] = SearchView(self._app)
 
         # empty view has already been created in self._setup_view starting at
         # View.ALBUM
@@ -300,29 +284,18 @@ class Window(Gtk.ApplicationWindow):
             "search-mode-active", self.views[View.ALBUM],
             "search-mode-active", GObject.BindingFlags.SYNC_CREATE)
 
-    @log
     def _select_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
-        if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self.props.active_view
-        else:
-            view = self.props.active_view.get_visible_child()
 
-        view.select_all()
+        self.props.active_view.select_all()
 
-    @log
-    def _select_none(self, action=None, param=None):
+    def _deselect_all(self, action=None, param=None):
         if not self.props.selection_mode:
             return
-        if self._headerbar.props.state == HeaderBar.State.MAIN:
-            view = self.props.active_view
-            view.unselect_all()
-        else:
-            view = self.props.active_view.get_visible_child()
-            view.select_none()
 
-    @log
+        self.props.active_view.deselect_all()
+
     def _on_key_press(self, widget, event):
         modifiers = event.get_state() & Gtk.accelerator_get_default_mod_mask()
         (_, keyval) = event.get_keyval()
@@ -372,7 +345,7 @@ class Window(Gtk.ApplicationWindow):
         # Ctrl+Shift+<KEY>
         elif modifiers == shift_ctrl_mask:
             if keyval == Gdk.KEY_A:
-                self._select_none()
+                self._deselect_all()
         # Alt+<KEY>
         elif modifiers == mod1_mask:
             # Go back from child view on Alt + Left
@@ -424,17 +397,14 @@ class Window(Gtk.ApplicationWindow):
                 and self._headerbar.props.state != HeaderBar.State.SEARCH):
             self._search.props.search_mode_active = True
 
-    @log
     def _on_back_button_pressed(self, gesture, n_press, x, y):
         self._headerbar.emit('back-button-clicked')
 
-    @log
     def _notify_mode_disconnect(self, data=None):
         self._player.stop()
         self.notifications_popup.terminate_pending()
         self._stack.disconnect(self._on_notify_model_id)
 
-    @log
     def _on_notify_mode(self, stack, param):
         previous_view = self._current_view
         self._current_view = self.props.active_view
@@ -456,14 +426,8 @@ class Window(Gtk.ApplicationWindow):
         ]
         allowed = self._current_view not in no_selection_mode
         self._headerbar.props.selection_mode_allowed = allowed
+        self._search_headerbar.props.selection_mode_allowed = allowed
 
-        # Disable renaming playlist if it was active when leaving
-        # Playlist view
-        if (previous_view == self.views[View.PLAYLIST]
-                and self.views[View.PLAYLIST].rename_active):
-            self.views[View.PLAYLIST].disable_rename_playlist()
-
-    @log
     def _toggle_view(self, view_enum):
         # TODO: The SEARCH state actually refers to the child state of
         # the search mode. This fixes the behaviour as needed, but is
@@ -474,7 +438,6 @@ class Window(Gtk.ApplicationWindow):
                 and not self._headerbar.props.state == HeaderBar.State.SEARCH):
             self._stack.set_visible_child(self.views[view_enum])
 
-    @log
     def _on_search_state_changed(self, klass, param):
         if (self._search.props.state != Search.State.NONE
                 or not self._view_before_search):
@@ -483,7 +446,6 @@ class Window(Gtk.ApplicationWindow):
         # Get back to the view before the search
         self._stack.set_visible_child(self._view_before_search)
 
-    @log
     def _switch_back_from_childview(self, klass=None):
         if self.props.selection_mode:
             return
@@ -495,13 +457,11 @@ class Window(Gtk.ApplicationWindow):
         if self._current_view in views_with_child:
             self._current_view._back_button_clicked(self._current_view)
 
-    @log
     def _on_selection_mode_changed(self, widget, data=None):
         if (not self.props.selection_mode
                 and self._player.state == Playback.STOPPED):
             self._player_toolbar.hide()
 
-    @log
     def _on_add_to_playlist(self, widget):
         if self.props.active_view == self.views[View.PLAYLIST]:
             return
@@ -519,7 +479,6 @@ class Window(Gtk.ApplicationWindow):
         self.props.selection_mode = False
         playlist_dialog.destroy()
 
-    @log
     def set_player_visible(self, visible):
         """Set PlayWidget action visibility
 
